package org.openstreetmap.josm.plugins.lanetool.data;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Observable;

import org.openstreetmap.josm.Main;
import org.openstreetmap.josm.command.Command;
import org.openstreetmap.josm.command.DeleteCommand;
import org.openstreetmap.josm.data.osm.OsmPrimitive;

/**
 * a lane template, concrete lanes can be way based or relation based
 */
abstract class AbstractLane<PrimitiveType extends OsmPrimitive> extends Observable implements Lane {
	
	protected final PrimitiveType osmObject;
	protected final LaneGroup parent;
	
	protected AbstractLane(PrimitiveType osmObject, LaneGroup parent) {
		assert osmObject != null;	
		assert parent != null;
				
		this.osmObject = osmObject;
		this.parent = parent;
	}
	
	public OsmPrimitive getOsmObject() {
		return osmObject;
	}
	
	public String getLaneType() {
		return osmObject.get("lane");
	}
	
	public void setLaneType(String value) {
		setTag("lane", value);
	}
	
	public boolean isSelected() {
		return Main.main.getCurrentDataSet().isSelected(osmObject);
	}
	
	public void select() {
		Main.main.getCurrentDataSet().setSelected(osmObject);
		Main.main.getCurrentDataSet().fireSelectionChanged();
	}
	
	public void addToSelection() {
		Collection<OsmPrimitive> selected = Main.main.getCurrentDataSet().getSelected();
		selected.add(osmObject);
		Main.main.getCurrentDataSet().setSelected(selected);
		Main.main.getCurrentDataSet().fireSelectionChanged();
	}
	
	public void removeFromSelection() {
		Collection<OsmPrimitive> selected = Main.main.getCurrentDataSet().getSelected();
		selected.remove(osmObject);
		Main.main.getCurrentDataSet().setSelected(selected);
		Main.main.getCurrentDataSet().fireSelectionChanged();
	}

	public Direction getDirection() {
		
		Direction direction = Direction.getDirection(osmObject);
		
		if (direction == null) {
			if (parent.getHighway() != null &&
					Direction.getDirection(parent.getHighway()) != null) {
				direction = Direction.getDirection(parent.getHighway());
			} else {
				direction = Direction.BOTH;
			}
		}
		
		return direction;
	}
	
	public void setDirection(Direction newDirection) {

		//TODO: how to handle directions that are implied? Remove tag or set explicit tag?
				
		if (osmObject == null) {
			removeTag("oneway");
		} else {
			switch (newDirection) {
			case FORWARD:
				setTag("oneway", "yes");
				break;
			case BACKWARD:
				setTag("oneway", "-1");
				break;
			case BOTH:
				if (parent.getHighway() == null
						|| Direction.getDirection(parent.getHighway()) == Direction.FORWARD 
						|| Direction.getDirection(parent.getHighway()) == Direction.BACKWARD) {
					setTag("oneway", "no");
				} else {
					removeTag("oneway");
				}
				break;
			}
		}
		
		setChanged(); //TODO: only for real changes?
		notifyObservers();
	}

	protected abstract void setTag(String key, String value);
	
	protected abstract void removeTag(String key);
	
	public RelationLane addSibling(String laneType, boolean toRight) {
			
		Integer ownPosition = parent.getLanePosition(this);
			
		assert ownPosition != null;
		
		int siblingPosition;
		if (toRight) {
			siblingPosition = ownPosition + 1;
		} else {
			siblingPosition = ownPosition - 1;
		}
		
		return parent.createAndInsertLane(laneType, siblingPosition, toRight);
	}
	
	/**
	 * removes this lane from its lane group and the dataset
	 */
	public void delete() {
				
		//remove from lane group
		
		parent.removeLane(this);
		
		//remove from dataset
				
		Collection<PrimitiveType> deleteCollection = new ArrayList<PrimitiveType>(1);
		deleteCollection.add(osmObject);
		Command deleteCommand = DeleteCommand.delete(Main.main.getEditLayer(), deleteCollection);
		
		if (deleteCommand != null) { //?: why can this be null (not documented)?
		
			deleteCommand.executeCommand();
			
			//change selection (remove osmObject from it if necessary)
			
			if (Main.main.getCurrentDataSet().getSelected().contains(osmObject)) {
				Collection<OsmPrimitive> selection = Main.main.getCurrentDataSet().getSelected();
				selection.remove(osmObject);
				Main.main.getCurrentDataSet().setSelected(selection);
			}
			
			Main.main.getCurrentDataSet().fireSelectionChanged();
			
		}
		
	}
	
}
