package org.openstreetmap.josm.plugins.lanetool.data;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import org.openstreetmap.josm.Main;
import org.openstreetmap.josm.command.AddCommand;
import org.openstreetmap.josm.command.ChangeCommand;
import org.openstreetmap.josm.command.Command;
import org.openstreetmap.josm.data.osm.OsmPrimitive;
import org.openstreetmap.josm.data.osm.Relation;
import org.openstreetmap.josm.data.osm.RelationMember;
import org.openstreetmap.josm.data.osm.Way;

public class LaneGroup extends Observable {

	private final Relation relation;
	
	private Map<Integer, Lane> laneMap; //TODO: handle multiple lanes with same integer or not-set-lanes
	
	/** creates a lane group. Also creates lanes from referenced OsmPrimitives. */
	LaneGroup(Relation relation) {		
		assert relation != null;
		assert "lane_group".equals(relation.get("type"));
		
		this.relation = relation;
		
		createLaneMap();
	}
	
	public Collection<Lane> getLanes() {
		return laneMap.values();
	}

	/**
	 * 
	 * @param laneType  may be null to indicate that a lane without any information is to be created
	 * @param position
	 * @param moveSublanesRight
	 * @return  the lane that has been created
	 */
	public RelationLane createAndInsertLane(String laneType, int position, boolean moveSublanesRight) {
		
		Relation laneRelation = new Relation();
		laneRelation.put("type", "lane");
		if (laneType != null) {
			laneRelation.put("lane", laneType);
		}
		
		RelationLane newLane = new RelationLane(laneRelation, this);
		
		AddCommand addCommand = new AddCommand(laneRelation);
		addCommand.executeCommand();
		
		insertLane(newLane, position, moveSublanesRight);
		
		return newLane;
	}
	
	private void insertLane(Lane child, int position, boolean moveSublanesRight) {
		
		Relation oldRelation = new Relation();
		oldRelation.cloneFrom(relation);
		
		//check whether existing sublanes have to be moved
		
		boolean moveSublanes = laneMap.containsKey(position);
		
		//update existing lane's positions
		
		if (moveSublanes) {

			Collection<RelationMember> membersToRemove = new ArrayList<RelationMember>();
			Collection<RelationMember> membersToAdd = new ArrayList<RelationMember>();
			
			for (RelationMember member : relation.getMembers()) {
				try {
					
					int memberPos = Integer.parseInt(member.getRole());
					int newMemberPos;
					
					if (memberPos >= position && moveSublanesRight) {
						newMemberPos = memberPos + 1;
					} else if (memberPos <= position && !moveSublanesRight) {
						newMemberPos = memberPos - 1;					
					} else {
						newMemberPos = memberPos;
					}
					
					if (newMemberPos != memberPos) {
						RelationMember newMember = new RelationMember(
								Integer.toString(newMemberPos), member.getMember());
						membersToRemove.add(member);
						membersToAdd.add(newMember);
					}
					
				} catch (NumberFormatException e) { }
			}

			if (membersToRemove.size() > 0) {
				List<RelationMember> memberList = relation.getMembers();
				memberList.removeAll(membersToRemove);
				memberList.addAll(membersToAdd);
				relation.setMembers(memberList);
				relation.setModified(true);
			}
			
		}		

		//add new child
		
		relation.addMember(new RelationMember(
				Integer.toString(position), child.getOsmObject()));
		
		//update map structure
		
		createLaneMap();
		
		//create and execute command, notify observers
		
		Command command = new ChangeCommand(oldRelation, relation);
		command.executeCommand();
		Main.main.getCurrentDataSet().fireSelectionChanged();

		setChanged();
		notifyObservers();
				
	}
	
	/**
	 * removes a lane from this group, but does not delete it
	 */
	Command removeLane(Lane lane) {
		
		Relation oldRelation = new Relation();
		oldRelation.cloneFrom(relation);
		
		for (Iterator<RelationMember> iterator = relation.getMembers().iterator(); iterator.hasNext();) {
			OsmPrimitive member = iterator.next().getMember();
			if (member.equals(relation)) {
				iterator.remove();
			}
		}
		
		return new ChangeCommand(oldRelation, relation);
		
	}
	
	/** returns position number of a lane; null if lane is not in this group */
	public Integer getLanePosition(Lane lane) {
		for (int pos : laneMap.keySet()) {
			if (laneMap.get(pos) == lane) {
				return pos;
			}
		}
		return null;
	}

	private void createLaneMap() {

		laneMap = new HashMap<Integer, Lane>();

		for(RelationMember member : relation.getMembers()) {

			try {

				int memberIndex;
				if (member.getRole().length() == 0) {
					memberIndex = 0; 
				} else {
					memberIndex = Integer.parseInt(member.getRole());
				}				
				
				Lane lane = null;
				
				if (member.getMember() instanceof Relation) {
					lane = new RelationLane((Relation)member.getMember(), this);
				} else if (member.getMember() instanceof Way){
					lane = new WayLane((Way)member.getMember(), this);
				}
				
				if (lane != null) {
					laneMap.put(memberIndex, lane);
				}
				//TODO: else? Error?

			} catch (NumberFormatException e) {
				// do not add this as child
			}

		}

	}
	
	Way getHighway() {
				
		List<Way> highwayMembers = new LinkedList<Way>();
		
		for (RelationMember member : relation.getMembers()) {
			if ("highway".equals(member.getRole())
					&& member.getMember() instanceof Way) {
				highwayMembers.add((Way)member.getMember());
			}
		}
		
		if (highwayMembers.size() != 1) {
			return null;
		} else {
			return highwayMembers.get(0);
		}
		
	}
	
	@Override
	public synchronized void addObserver(Observer o) {
		super.addObserver(o);
		for (Lane lane : laneMap.values()) {
			lane.addObserver(o);
		}
	}
	
}
