package org.openstreetmap.josm.plugins.lanetool.view;

import static org.openstreetmap.josm.plugins.lanetool.data.Direction.BACKWARD;
import static org.openstreetmap.josm.plugins.lanetool.data.Direction.BOTH;
import static org.openstreetmap.josm.plugins.lanetool.data.Direction.FORWARD;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;

import javax.swing.JPopupMenu;

import org.openstreetmap.josm.Main;
import org.openstreetmap.josm.plugins.lanetool.data.Lane;
import org.openstreetmap.josm.plugins.lanetool.view.style.LanesStyle;
import org.openstreetmap.josm.plugins.lanetool.view.util.ImageUtil;

/**
 * view representation of a lane
 */
class LaneView {
	
	private Lane lane;
	
	private LanesStyle style;
	
	private Rectangle laneRect;
	private Rectangle directionRect;

	private int xOffset;
	private int yMin;
	private int yMax;
	
	public void setXOffset(int xOffset) {
		this.xOffset = xOffset;
		determineRects();
	}

	public LaneView(Lane visualizedLane, LanesStyle style, int yMin, int yMax) {
		
		this.lane = visualizedLane;
		this.style = style;
		
		this.yMin = yMin;
		this.yMax = yMax;
				
		setXOffset(0);
		
	}
	
	private void determineRects() {

		Image directionImage = style.getDirectionImage(lane);
		
		int directionRectHeight = directionImage.getHeight(null);
		
		directionRect = new Rectangle(
				xOffset - directionImage.getWidth(null)/2,
				0,
				directionImage.getWidth(null),
				directionRectHeight);
		
		laneRect = new Rectangle(
				xOffset - getLaneWidth()/2,
				yMin + directionRectHeight,
				getLaneWidth(),
				(yMax-yMin));
		
	}
	
	int getLaneWidth() {
		
		return style.getLaneImage(lane).getWidth(null);

	}
	
	/**
	 * handles a left click on the lanes panel by checking whether this lane is affected
	 * 
	 * @return  true if the click was handled
	 */
	public boolean handleLeftClick(Component invoker, int x, int y, boolean shift, boolean ctrl) {

		if (directionRect.contains(x, y)) {
		
			if (!shift) {
				switch(lane.getDirection()) {
				case FORWARD: lane.setDirection(BACKWARD); break;
				case BACKWARD: lane.setDirection(BOTH); break;
				case BOTH: lane.setDirection(FORWARD); break;
				}
			} else {
				switch(lane.getDirection()) {
				case FORWARD: lane.setDirection(BOTH); break;
				case BACKWARD: lane.setDirection(FORWARD); break;
				case BOTH: lane.setDirection(BACKWARD); break;
				}
			}

			return true;
		
		} else if (laneRect.contains(x, y)) {
			
			if (shift) {
				lane.addToSelection();
			} else if (ctrl) {
				lane.removeFromSelection();
			} else {
				lane.select();
			}			
			
			return true;
			
		} else {
			
			return false;
			
		}
		
	}
	
	/**
	 * handles a right click on the lanes panel by checking whether this lane is affected
	 * 
	 * @return  true if the click was handled
	 */
	public boolean handleRightClick(Component invoker, int x, int y, boolean shift, boolean ctrl) {
		
		if (laneRect.contains(x, y)) {
			
			JPopupMenu popup = new LanePopupMenu(lane);
			popup.show(invoker, x, y);
			
			return true;
		
		} else {
			
			return false;
			
		}
		
	}
	
	/**
	 * recursively paints this lane and all sublanes
	 */
	public void paint(Graphics g) {

		Image laneImage = style.getLaneImage(lane);

		ImageUtil.fillWithImage(laneImage, g, laneRect);

		Image directionImage = style.getDirectionImage(lane);

		ImageUtil.fillWithImage(directionImage, g, directionRect);
		
		if (lane.isSelected()) {
			Color c = Main.pref.getColor("selection", Color.YELLOW);
			c = new Color(c.getRed(), c.getGreen(), c.getBlue(), 128);
			g.setColor(c); //?: where do I get a list of names from?
			g.fillRect(laneRect.x, laneRect.y, laneRect.width, laneRect.height);
		}
		
	}
		
}
